#ifndef __GEDIT_BOOKMARKS_MESSAGES_MESSAGES_H__
#define __GEDIT_BOOKMARKS_MESSAGES_MESSAGES_H__

#include "gedit-bookmarks-message-add.h"
#include "gedit-bookmarks-message-goto-next.h"
#include "gedit-bookmarks-message-goto-previous.h"
#include "gedit-bookmarks-message-remove.h"
#include "gedit-bookmarks-message-toggle.h"

#endif /* __GEDIT_BOOKMARKS_MESSAGES_MESSAGES_H__ */
